/* ============================================================
 *
 * This file is a part of KDE project
 *
 *
 * Date        : 2012-01-08
 * Description : Kipi-Plugins Git version
 *
 * Copyright (C) 2012 by Andi Clemens <andi dot clemens at googlemail dot com>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include "kpversion.h"
#include "gitversion.h"

namespace pluginsVersion
{

const QString gitVersion()
{
    return QString::fromUtf8(GITVERSION);
}

} // namespace pluginsVersion
