/* ============================================================
*
* This file is a part of kipi-plugins project
*
*
* Date        : 2012-02-02
* Description : a plugin to export photos or videos to ImageShack web service
*
* Copyright (C) 2012 Dodon Victor <dodonvictor at gmail dot com>
*
* This program is free software; you can redistribute it
* and/or modify it under the terms of the GNU General
* Public License as published by the Free Software Foundation;
* either version 2, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* ============================================================ */

#ifndef IMAGESHACKITEM_H
#define IMAGESHACKITEM_H

namespace KIPIImageshackPlugin
{

class ImageshackGallery
{

public:

    ImageshackGallery()
    {
    }

    QString m_user;
    QString m_server;
    QString m_name;
    QString m_title;
    QString m_url;
};

// -------------------------------------------------------------------

class ImageshackPhoto
{

public:

    ImageshackPhoto()
    {
    }

    QString m_server;
    QString m_name;
    QString m_bucket;
};

} // namespace KIPIImageshackPlugin

#endif // IMAGESHACKITEM_H
